local STI = require 'libs.STI'
local HC = require 'libs.HC'


local g = love.graphics

local TILE_S = 16

local Map = {}
Map.__index = Map

function Map.create(path)
    local self = setmetatable({}, Map)
    self.collider = HC.new(5)
    return self
end

function Map:load(path)
    self.map = STI.new(path, { "hc" })
    self.map:hc_init(self.collider)
    for _, layer in ipairs(self.map.layers) do
        if layer.type == 'objectgroup' then
            for _, object in ipairs(layer.objects) do
                if object.type == 'SpawnPoint' then
                    self.spawnPoint = { x=(object.x), y=(object.y)}
                end
            end
        end
    end
end

function Map:getSpawnPoint()
    return self.spawnPoint
end

function Map:getBounds()
    local firstLayer = self.map.layers[1]
    return { x=firstLayer.x, y=firstLayer.y, width=firstLayer.width * TILE_S, height=firstLayer.height * TILE_S }
end

function Map:draw(x, y, width, height)
    -- Draw Range culls unnecessary tiles
    self.map:setDrawRange(x, y, width, height)

    -- Draw the map and all objects within
    self.map:draw()

    -- Draw Collision Map (useful for debugging)
    g.setColor(255, 0, 0, 255)
    self.map:hc_draw(self.collider)

    -- Reset color
    g.setColor(255, 255, 255, 255)
end

function Map:update(dt)
    self.map:update(dt)
end

function Map:getCollider()
    return self.collider
end

return Map
